/*
 * Copyright (c) 1997 David J. Wilson.
 * All rights reserved.
 *
 * Copyright (c) 1997 Philip R. Wilson.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/types.h>

#include <stdio.h>
#ifdef __FreeBSD__
#include <string.h>
#endif
#include <termios.h>
#include <unistd.h>
#include "ansi_io.h"
  
#ifndef lint
static const char rcsid[] = "$Id: s_tty.c,v 1.2 1997/06/09 06:48:22 prw Exp $";
#endif /* not lint */

#ifdef __linux__
struct termio t_save;
struct termio ttyswitch;
#endif

#ifdef __FreeBSD__
struct termios t_save;
struct termios ttyswitch;
#endif

int timeout_val = -1; /* -1 hang endlessly, 0 no wait, + wait */


int echo_tog(toggle echo)
{
#ifdef __linux__
	if (ioctl(0, TCGETA, &ttyswitch) == -1) {
#endif
#ifdef __FreeBSD__
	if (ioctl(0, TIOCGETA, &ttyswitch) == -1) {
#endif
		if (DEBUG)
			fprintf(stderr, 
			"libansi_io: %s(): stty failed: not a terminal\n",
			__FUNCTION__);
		return (0);
	}
	switch (echo) {
		case off:
			ttyswitch.c_lflag &= ~(ECHO|ECHONL);
			break;
		case on:
			ttyswitch.c_lflag |= ECHO;	
			break;
		default:
			if (DEBUG)
				fprintf(stderr, 
				"libansi_io: got %d echo value, failing...", echo);
			return (0);
	}

#ifdef __linux__
    	if (ioctl(0, TCSETAF, &ttyswitch) == -1) {
#endif
#ifdef __FreeBSD__
	if (ioctl(0, TIOCSETAF, &ttyswitch) == -1) {
#endif
		if (DEBUG)
			fprintf(stderr, "libansi_io: echo mode failed\n");
        	return (0);
	}

	return (1);
}


int cbreak(toggle cb)
{
#ifdef __linux__
        if (ioctl(0, TCGETA, &ttyswitch) == -1) {
#endif
#ifdef __FreeBSD__
	if (ioctl(0, TIOCGETA, &ttyswitch) == -1) {
#endif
                if (DEBUG)
                        fprintf(stderr,
			"debug: %s(): stty failed: not a terminal\n",
                         __FUNCTION__);
                return (0);
        }
        switch (cb) {
                case on:
                    	ttyswitch.c_lflag &= ~ICANON;
			ttyswitch.c_lflag |= ISIG;
			ttyswitch.c_cc[VMIN] = 1;
			ttyswitch.c_cc[VTIME] = 0;
                        break;
                case off:
                        ttyswitch.c_lflag |= ICANON;
			ttyswitch.c_lflag &= ~ISIG;
                        break;
                default:
                        if (DEBUG)
                                fprintf(stderr,
				 "libansi_io: got %d echo value, failing...", cb); 
			return(false);
        }

#ifdef __linux__
        if (ioctl(0, TCSETAF, &ttyswitch) == -1) {
#endif
#ifdef __FreeBSD__
	if (ioctl(0, TIOCSETAF, &ttyswitch) == -1) {
#endif

                if (DEBUG)
                        fprintf(stderr, "libansi_io: sb mode failed\n");
                return(0);
        }

        return (1);
}


int aio_on(void) 
{
        setvbuf(stdin, (char *)NULL, _IONBF, 0);

#ifdef __linux__
        if (ioctl(0, TCGETA, &t_save) == -1) {
#endif
#ifdef __FreeBSD__
	if (ioctl(0, TIOCGETA, &t_save) == -1) {
#endif
		if (DEBUG) 
                        fprintf(stderr, 
			"libansi_io: %s(): stty failed: not a terminal\n",
			__FUNCTION__);
		return (0);
	}

	high_ascii(true);

	return (1);
}	

	
void high_ascii(bool tog)
{
	switch (tog) {
		case true:
			aio_write("\e(U");
			break;
		case false:
			aio_write("\e(B");
			break;
		default:
			break;
	}
}


void aio_off(void) 
{ 
	high_ascii(false);

#ifdef __linux__
	ioctl(0, TCSETAF, &t_save); 
#endif
#ifdef __FreeBSD__
	ioctl(0, TIOCSETAF, &t_save);
#endif

}


void getch_timeout(int timeout)
{
	if (timeout <= -1)
		timeout_val = -1;
	else
		timeout_val = timeout;
}


int getch(void)
{
	int nfds;
	fd_set readfds;
	struct timeval tv;
	int ch = 0;

	FD_ZERO(&readfds);
	tv.tv_sec = 0;
	tv.tv_usec = timeout_val;
	
	FD_SET(0, &readfds);
	if (timeout_val < 0)
		nfds = select(1, &readfds, NULL, NULL, NULL);
	else
		nfds = select(1, &readfds, NULL, NULL, &tv);

	if (nfds) 
		ch = getchar();
	else 
		return(ERR); 
	return (ch);
}


void clear_screen(void)
{
        aio_write("\033[2J\033[1;1H");
}
